package com.gct.tool.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.common.util.SGTUtil;
import com.gct.tool.entity.aoid.AoidSinglegtYield;
import com.gct.tool.entity.excel.LiqAnalyseData;
import com.gct.tool.entity.excel.SingleGtData;
import com.gct.tool.service.aoid.AoidDailyYieldService;
import com.gct.tool.service.aoid.AoidSinglegtYieldService;
import com.gct.tool.service.centralbase.CbCdWellSourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author xusirui 2022/9/28
 */
@Service
public class DataAnalyseService implements ApplicationRunner {

    @Value("${gt-total}")
    Integer gtTotal;

    @Value("${begin-date}")
    String beginDate;

    @Value("${end-date}")
    String endDate;

    @Autowired
    AoidDailyYieldService aoidDailyYieldService;

    @Autowired
    CbCdWellSourceService cbCdWellSourceService;

    @Autowired
    AoidSinglegtYieldService aoidSinglegtYieldService;

    private final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

    private final String excelFileBasePath = System.getProperty("user.dir") + "/.excel/";

    public DataAnalyseService() {
        File file = new File(excelFileBasePath);
        if (!file.exists()) file.mkdirs();
    }

    public void produceLiqAnalyseTotal(){
        for(LocalDate date = LocalDate.parse(beginDate);date.isBefore(LocalDate.parse(endDate));date = date.plusMonths(1)){
            produceLiqAnalyseMonth(date);
        }
    }

    public List<String> readWellIds(){
        try{
            ClassPathResource resource = new ClassPathResource("wellIds.txt");
            BufferedReader br = new BufferedReader(new InputStreamReader(resource.getInputStream()));
            String wellId;
            List<String> wellIds = new ArrayList<>();
            while ((wellId = br.readLine())!=null){
                wellIds.add(wellId);
            }
            return wellIds;
        }catch (Exception e){
            e.printStackTrace();
        }
        return new ArrayList<>();
    }

    public void produceLiqAnalyseMonth(LocalDate beginDate) {
        String fileName = excelFileBasePath + "功图数据-" + beginDate.format(DateTimeFormatter.ofPattern("yyyy-MM")) + ".xlsx";
        File file = new File(fileName);
        if (!file.exists()) {
            try {
                file.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        ExcelWriter excelWriter = EasyExcel.write(fileName, SingleGtData.class).build();
        List<String> wellIds = readWellIds();
        for (LocalDate date = beginDate;date.isBefore(beginDate.plusMonths(1));date = date.plusDays(1)){
            System.out.println(date.format(DateTimeFormatter.ISO_LOCAL_DATE));
            final LocalDate produceDate = date;
                WriteSheet writeSheet = EasyExcel.writerSheet("第二采油作业区").build();
                List<SingleGtData> singleGtDatas = aoidSinglegtYieldService.list(new QueryWrapper<AoidSinglegtYield>()
                        .eq("prod_date::date", produceDate)
                        .in("well_id",wellIds))
                        .stream()
                        .filter(aoidSinglegtYield -> aoidSinglegtYield.getSgtIsLegal() == 1)
                        .map(aoidSinglegtYield -> {
                            SingleGtData singleGtData = new SingleGtData();
                            singleGtData.setProdDate(aoidSinglegtYield.getProdDate().format(formatter));
                            singleGtData.setWorkDiagram(aoidSinglegtYield.getSgt());
                            double[][] sgt = SGTUtil.encodeToUseArray(aoidSinglegtYield.getSgt());
                            double[] maxLoadP = Arrays.stream(sgt).max(Comparator.comparingDouble(x -> x[1])).get();
                            double[] minLoadP = Arrays.stream(sgt).min(Comparator.comparingDouble(x -> x[1])).get();
                            singleGtData.setMaxLoad(maxLoadP[1]);
                            singleGtData.setMinLoad(minLoadP[1]);
                            singleGtData.setWellId(aoidSinglegtYield.getWellId());
                            singleGtData.setAnalyseResult(aoidSinglegtYield.getEczdjg());
                            double s = 0;
                            for (int i = 0; i < sgt.length / 2 - 1; i++) {
                                double h = Math.abs(sgt[i][0] - sgt[i + 1][0]);
                                double Si = (sgt[i][1] + sgt[i + 1][1]) * h / 2.0;
                                s += Si;
                            }
                            for (int i = sgt.length / 2 - 1; i < sgt.length - 1; i++) {
                                double h = Math.abs(sgt[i][0] - sgt[i + 1][0]);
                                double Si = (sgt[i][1] + sgt[i + 1][1]) * h / 2.0;
                                s -= Si;
                            }
                            singleGtData.setWorkDiagramArea(s);
                            return singleGtData;
                        }).collect(Collectors.toList());
                excelWriter.write(singleGtDatas, writeSheet);
        }
        excelWriter.finish();
    }


    @Override
    public void run(ApplicationArguments args) throws Exception {
        produceLiqAnalyseTotal();
        System.exit(0);
    }
}
